==> RSTouchscreenNG-1.4.11.zip: RSTouchscreenNG/projects/pages/src/lib/gpio-actions.component.ts <==
/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from "@angular/core";
import {PrinterState, PrintersState, GPIO, ConnectionHandler, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {RSIntKeyboard, RSModal} from "front-ui";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-gpio-actions",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-empty-sideicon></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-empty-sideicon></rs-empty-sideicon>
      </rs-side>
      <main>
        <rs-headline><i class="rs rs-gpio"></i> <?php _('GPIO Actions') ?></rs-headline>
        <div class="content-h1" rsDrag>
          <div class="valuelist center-icons">
            <div class="button-icon sep-icon" *ngFor="let f of gpiosFiltered" style="min-width:12em">
              <rs-button class="secondary outline full" (hit)="runGPIO(f)">
                <!-- <i *ngIf="!f.icon" class="fa {{f.operation===4 ? 'fa-tachometer' : (f.state ? 'fa-toggle-on' : 'fa-toggle-off')}}"></i>
                <i *ngIf="f.icon" class="rs-fw rs-bigtab" style="display: inline-block;vertical-align: baseline;"><img style="fill:#ffffff" [src]="f.state ? f.icon : f.iconOff"></i> -->
                <rs-dyn-icon [icon]="(f.icon ? (f.state ? f.icon : (f.iconOff ? f.iconOff : f.icon)) : 'fa '+ (f.operation===4 ? 'fa-tachometer' : (f.state ? 'fa-toggle-on' : 'fa-toggle-off')))"></rs-dyn-icon>
                <h3 class="text" style="padding:8px">{{f.display}}<span *ngIf="f.operation===4"> - {{(f.state ? (f.pwmDutyCycle | fixed:0) + '%' : "<?php _('Off') ?>
                  ")}}</span></h3></rs-button>
            </div>
          </div>
          <div *ngIf="gpiosFiltered.length===0" class="alert info margin">
            <?php _('No predefined gpio actions defined!') ?>
          </div>
        </div>
      </main>
    </rs-page>
  `
})
export class GPIOActionsComponent implements OnDestroy {
  printer: PrinterState;
  active: PrinterState = undefined;
  printers$: Observable<PrintersState>;
  gpios$: Observable<GPIO[]>;
  private gpiosSubscription: SubscriptionLike;
  gpiosFiltered: GPIO[] = [];
  gpios: GPIO[] = [];

  private printersSubscription: SubscriptionLike;

  constructor(private intKbd: RSIntKeyboard,
              private connectionHandler: ConnectionHandler,
              private store: Store<RepetierState>,
              public modal: RSModal) {
    this.printers$ = store.select("printers");
    this.gpios$ = store.select(state => state.server.gpios);
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.printer = p.printers[p.active];
      } else {
        this.printer = undefined;
      }
      this.filterGPIO();
    });
    this.gpiosSubscription = this.gpios$.subscribe((w: GPIO[]) => {
      this.gpios = w;
      this.filterGPIO();
    });
  }

  filterGPIO() {
    const a: GPIO[] = [];
    this.gpios.forEach(x => {
      if (x.showInMenu && ((this.printer && x.slug === this.printer.slug) || x.slug === "")) {
        a.push(x);
      }
    });
    this.gpiosFiltered = a;
  }

  runGPIO(cmd: GPIO) {
    if (cmd.operation === 4) {
      this.intKbd.getInt(
        cmd.pwmDutyCycle,
        cmd.display,
        {unit: "%", min: 0, max: 100}).then((data) => {
        if (data === 0) {
          this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "off"});
        } else {
          this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "" + data});
        }
      });
    } else {
      if (cmd.securityQuestion) {
        this.modal.confirm("<?php _('Security Question') ?>",
          (cmd.state ? "<?php _('Really disable @?') ?>" : "<?php _('Really enable @?') ?>").replace("@", cmd.display), {}).then(
          () => this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "toggle"}), () => {
          }
        );
      } else {
        this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "toggle"});
      }
    }
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
    this.gpiosSubscription.unsubscribe();
  }

  protected readonly undefined = undefined;
}

==> RSTouchscreenNG-1.4.16.zip: RSTouchscreenNG/projects/pages/src/lib/gpio-actions.component.ts <==
/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, OnDestroy} from "@angular/core";
import {PrinterState, PrintersState, GPIO, ConnectionHandler, RepetierState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {RSIntKeyboard, RSModal} from "front-ui";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-gpio-actions",
  template: `
    <rs-page>
      <rs-side>
        <rs-sideicon title="<?php _('Menu') ?>" icon="rs-menu" link="/"></rs-sideicon>
      </rs-side>
      <rs-side>
        <rs-empty-sideicon></rs-empty-sideicon>
      </rs-side>
      <rs-side>
        <rs-empty-sideicon></rs-empty-sideicon>
      </rs-side>
      <main>
        <rs-headline><i class="rs rs-gpio"></i> <?php _('GPIO Actions') ?></rs-headline>
        <div class="content-h1" rsDrag>
          <div>
            <div class="valuelist center-icons margin-leftright">
              <div class="button-icon sep-icon" *ngFor="let f of gpiosFiltered" style="min-width:12em">
                <rs-button class="secondary outline full" (hit)="runGPIO(f)">
                  <!-- <i *ngIf="!f.icon" class="fa {{f.operation===4 ? 'fa-tachometer' : (f.state ? 'fa-toggle-on' : 'fa-toggle-off')}}"></i>
                  <i *ngIf="f.icon" class="rs-fw rs-bigtab" style="display: inline-block;vertical-align: baseline;"><img style="fill:#ffffff" [src]="f.state ? f.icon : f.iconOff"></i> -->
                  <rs-dyn-icon [icon]="(f.icon ? (f.state ? f.icon : (f.iconOff ? f.iconOff : f.icon)) : 'fa '+ (f.operation===4 ? 'fa-tachometer' : (f.state ? 'fa-toggle-on' : 'fa-toggle-off')))"></rs-dyn-icon>
                  <h3 class="text" style="padding:8px">{{ f.display }}<span *ngIf="f.operation===4"> - {{(f.state ? (f.pwmDutyCycle | fixed:0) + '%' : "<?php _('Off') ?>
                    ")}}</span></h3></rs-button>
              </div>
            </div>
            <div *ngIf="gpiosFiltered.length===0" class="alert info margin">
              <?php _('No predefined gpio actions defined!') ?>
            </div>
          </div>
        </div>
      </main>
    </rs-page>
  `
})
export class GPIOActionsComponent implements OnDestroy {
  printer: PrinterState;
  active: PrinterState = undefined;
  printers$: Observable<PrintersState>;
  gpios$: Observable<GPIO[]>;
  private gpiosSubscription: SubscriptionLike;
  gpiosFiltered: GPIO[] = [];
  gpios: GPIO[] = [];

  private printersSubscription: SubscriptionLike;

  constructor(private intKbd: RSIntKeyboard,
              private connectionHandler: ConnectionHandler,
              private store: Store<RepetierState>,
              public modal: RSModal) {
    this.printers$ = store.select("printers");
    this.gpios$ = store.select(state => state.server.gpios);
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.printer = p.printers[p.active];
      } else {
        this.printer = undefined;
      }
      this.filterGPIO();
    });
    this.gpiosSubscription = this.gpios$.subscribe((w: GPIO[]) => {
      this.gpios = w;
      this.filterGPIO();
    });
  }

  filterGPIO() {
    const a: GPIO[] = [];
    this.gpios.forEach(x => {
      if (x.showInMenu && ((this.printer && x.slug === this.printer.slug) || x.slug === "")) {
        a.push(x);
      }
    });
    this.gpiosFiltered = a;
  }

  runGPIO(cmd: GPIO) {
    if (cmd.operation === 4) {
      this.intKbd.getInt(
        cmd.pwmDutyCycle,
        cmd.display,
        {unit: "%", min: 0, max: 100}).then((data) => {
        if (data === 0) {
          this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "off"});
        } else {
          this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "" + data});
        }
      });
    } else {
      if (cmd.securityQuestion) {
        this.modal.confirm("<?php _('Security Question') ?>",
          (cmd.state ? "<?php _('Really disable @?') ?>" : "<?php _('Really enable @?') ?>").replace("@", cmd.display), {}).then(
          () => this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "toggle"}), () => {
          }
        );
      } else {
        this.connectionHandler.send("GPIORun", {uuid: cmd.uuid, cmd: "toggle"});
      }
    }
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
    this.gpiosSubscription.unsubscribe();
  }

  protected readonly undefined = undefined;
}
